/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2010 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package org.ow2.petals.commons.threadlocal;

import org.ow2.easywsdl.extensions.wsdl4complexwsdl.WSDL4ComplexWsdlFactory;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlException;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlReader;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlWriter;

/**
 * <p>
 * WSDL 4 Complex WSDLs readers and writers.
 * </p>
 * <p>
 * To avoid several instance of the same WSDL4ComplexWsdl writer and reader in
 * the same Thread, each WSDL4ComplexWsdl must be created using these factories.
 * </p>
 * 
 * @author rnaudin - EBM Websourcing
 */
public class WSDL4ComplexWsdls {

    /**
     * <p>
     * {@link WSDL4ComplexWsdlReader} as thread local.
     * </p>
     * <p>
     * The WSDL4ComplexWsdl Reader is the default one provided by the current classloader
     * (no property set).
     * </p>
     */
    private final static ThreadLocal<WSDL4ComplexWsdlReader> defaultWSDL4ComplexWsdlReaderThreadLocal = new ThreadLocal<WSDL4ComplexWsdlReader>() {

        @Override
        protected WSDL4ComplexWsdlReader initialValue() {
            try {
                return WSDL4ComplexWsdlFactory.newInstance().newWSDLReader();
            } catch (WSDL4ComplexWsdlException e) {
                throw new RuntimeException("Failed to create WSDL4ComplexWsdl Reader", e);
            }
        }
    };

    /**
     * <p>
     * {@link WSDL4ComplexWsdlWriter} as thread local.
     * </p>
     * <p>
     * The WSDL4ComplexWsdl Writer is the default one provided by the classpath
     * (no property set).
     * </p>
     */
    private final static ThreadLocal<WSDL4ComplexWsdlWriter> defaultWSDL4ComplexWsdlWriterThreadLocal = new ThreadLocal<WSDL4ComplexWsdlWriter>() {

        @Override
        protected WSDL4ComplexWsdlWriter initialValue() {
            try {
                return WSDL4ComplexWsdlFactory.newInstance().newWSDLWriter();
            } catch (WSDL4ComplexWsdlException e) {
                throw new RuntimeException("Failed to create WSDL4ComplexWsdl Writer", e);
            }
        }
    };

    /**
     * <p>
     * Get {@link WSDL4ComplexWsdlReader} as thread local.
     * </p>
     * <p>
     * The WSDL4ComplexWsdl Reader is the default one provided by the classpath
     * (no property set).
     * </p>
     */
    public static final WSDL4ComplexWsdlReader getDefaultReader() {
        return defaultWSDL4ComplexWsdlReaderThreadLocal.get();
    }

    /**
     * <p>
     * Get {@link WSDL4ComplexWsdlWriter} as thread local.
     * </p>
     * <p>
     * The WSDL4ComplexWsdl Writer is the default one provided by the classpath
     * (no property set).
     * </p>
     */
    public static final WSDL4ComplexWsdlWriter getDefaultWriter() {
        return defaultWSDL4ComplexWsdlWriterThreadLocal.get();
    }

}
